<?php
namespace nur\sery\wip\schema\_assoc;

use nulib\cl;
use nulib\ref\schema\ref_schema;
use nulib\ValueException;
use nur\sery\wip\schema\Schema;
use nur\sery\wip\schema\Wrapper;

/**
 * Class AssocSchema
 */
class AssocSchema extends Schema {
  /** @var array meta-schema d'un schéma de nature tableau associatif */
  const METASCHEMA = ref_schema::ASSOC_METASCHEMA;

  /**
   * indiquer si $definition est une définition de schéma de nature tableau
   * associatif que {@link normalize_definition()} pourrait normaliser
   */
  static function isa_definition($definition): bool {
    if (!is_array($definition)) return false;
    # nature explicitement spécifiée
    if (self::have_nature($definition, $nature)) {
      return $nature === "assoc";
    }
    # tableau associatif
    return !cl::have_num_keys($definition);
  }

  static function normalize_definition($definition, $definitionKey=null): array {
    if (!is_array($definition)) $definition = [$definition];
    if (!self::have_nature($definition)) {
      $definition = [
        "?array",
        "" => "assoc",
        "schema" => $definition,
      ];
    }
    self::_normalize_definition($definition, $definitionKey);
    self::_ensure_nature($definition, "assoc", "array");
    return $definition;
  }

  function __construct($definition=null, $definitionKey=null, bool $normalize=true) {
    if ($definition === null) $definition = static::SCHEMA;
    if ($normalize) {
      $definition = self::normalize_definition($definition, $definitionKey);
      $this->_definition = $definition;
      self::_ensure_type($definition);
      self::_ensure_schema_instances($definition);
    }
    $this->definition = $definition;
    $keys = [];
    foreach ($definition["schema"] as $key => $schema) {
      if (!$schema["computed"]) $keys[] = $key;
    }
    $this->keys = $keys;
  }

  function isAssoc(?AssocSchema &$schema=null): bool {
    $schema = $this;
    return true;
  }

  protected array $keys;

  function getKeys(): array {
    return $this->keys;
  }

  function getSchema($key): Schema {
    if ($key === null) return $this;
    $schema = $this->definition["schema"][$key] ?? null;
    if ($schema === null) throw ValueException::invalid_key($key);
    return $schema;
  }

  protected function newWrapper(): AssocWrapper {
    return new AssocWrapper($this);
  }

  function getWrapper(&$value=null, $valueKey=null, ?array $params=null, ?Wrapper &$wrapper=null): AssocWrapper {
    # si pas de valeur ni de wrapper, pas de vérification et donc pas d'exception
    # cf le code similaire dans ScalarWrapper::__construct()
    $verifix = $value !== null || $wrapper !== null;
    if (!($wrapper instanceof AssocWrapper)) $wrapper = $this->newWrapper();
    if ($params !== null) $wrapper->resetParams($params);
    return $wrapper->reset($value, $valueKey, $verifix);
  }
}
