<?php
namespace nur\sery\wip\schema\_assoc;

use nulib\ValueException;
use nur\sery\wip\schema\_scalar\ScalarResult;
use nur\sery\wip\schema\input\Input;
use nur\sery\wip\schema\types\IType;
use nur\sery\wip\schema\Wrapper;
use nur\sery\wip\schema\WrapperContext;

class AssocWrapper extends Wrapper {
  function __construct(AssocSchema $schema, &$array=null, $arrayKey=null, ?array $params=null) {
    $keys = $schema->getKeys();
    $keyTypes = [];
    $keyWrappers = [];
    foreach ($keys as $key) {
      $keyTypes[$key] = null;
      $keyWrappers[$key] = $schema->getSchema($key)->getWrapper();
    }
    $this->context = $context = new AssocWrapperContext($schema, null, null, $params);
    $context->arrayResult = new ScalarResult();
    $context->keys = $keys;
    $context->keyTypes = $keyTypes;
    $context->keyWrappers = $keyWrappers;
    $context->result = new AssocResult($context);

    $throw = $context->throw;
    if ($array !== null && $throw === null) {
      # Si $value est null, ne pas lancer d'exception, parce qu'on considère que
      # c'est une initialisation sans conséquences
      $throw = true;
    }
    $context->throw = $throw ?? false;
    $this->reset($array, $arrayKey);
    $context->throw = $throw ?? true;
  }

  function isAssoc(?AssocWrapper &$wrapper=null): bool { $wrapper = $this; return true; }

  /** @var AssocWrapperContext  */
  protected WrapperContext $context;

  function reset(&$array, $arrayKey=null, ?array $params=null): Wrapper {
    if ($array instanceof Input) $input = $array;
    else $input = $this->newInput($array);
    $context = $this->context;
    $context->input = $input;
    $context->valueKey = $arrayKey;
    $this->afterModify($params);
    return $this;
  }

  function getKeys(): array {
    return $this->context->keys;
  }

  /** @param string|int|null $key */
  function select($key=null): Wrapper {
    #XXX il faut que context contiennent les informations pour la clé sélectionnée
    if ($key === null) return $this;
    $wrapper = $this->context->keyWrappers[$key] ?? null;
    if ($wrapper === null) throw ValueException::invalid_key($key);
    return $wrapper;
  }

  function isNormalized(): bool {
    return $this->context->result->normalized;
  }

  function ensureKeys(): bool {
  }
  function orderKeys(): bool {
  }
}
